@@ -1,6 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -10,7 +10,6 @@ __docformat__ = 'restructuredtext en'
 '''The main GUI'''
 
 import collections, os, sys, textwrap, time, gc, errno, re
-from Queue import Queue, Empty
 from threading import Thread
 from collections import OrderedDict
 from io import BytesIO
@@ -22,7 +21,7 @@ from PyQt5.Qt import (
 
 from calibre import prints, force_unicode, detect_ncpus
 from calibre.constants import (
-        __appname__, isosx, iswindows, filesystem_encoding, DEBUG)
+        __appname__, isosx, iswindows, filesystem_encoding, DEBUG, config_dir)
 from calibre.utils.config import prefs, dynamic
 from calibre.utils.ipc.pool import Pool
 from calibre.db.legacy import LibraryDatabase
@@ -31,6 +30,7 @@ from calibre.gui2 import (error_dialog, GetMetadata, open_url,
         gprefs, max_available_height, config, info_dialog, Dispatcher,
         question_dialog, warning_dialog)
 from calibre.gui2.cover_flow import CoverFlowMixin
+from calibre.gui2.changes import handle_changes
 from calibre.gui2.widgets import ProgressIndicator
 from calibre.gui2.update import UpdateMixin
 from calibre.gui2.main_window import MainWindow
@@ -51,6 +51,9 @@ from calibre.gui2.job_indicator import Pointer
 from calibre.gui2.dbus_export.widgets import factory
 from calibre.gui2.open_with import register_keyboard_shortcuts
 from calibre.library import current_library_name
+from calibre.srv.library_broker import GuiLibraryBroker
+from polyglot.builtins import unicode_type, string_or_bytes
+from polyglot.queue import Queue, Empty
 
 
 class Listener(Thread):  # {{{
@@ -85,11 +88,8 @@ class Listener(Thread):  # {{{
 # }}}
 
 
-_gui = None
-
-
 def get_gui():
-    return _gui
+    return getattr(get_gui, 'ans', None)
 
 
 def add_quick_start_guide(library_view, refresh_cover_browser=None):
@@ -136,7 +136,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     shutting_down = False
 
     def __init__(self, opts, parent=None, gui_debug=None):
-        global _gui
         MainWindow.__init__(self, opts, parent=parent, disable_automatic_gc=True)
         self.setWindowIcon(QApplication.instance().windowIcon())
         self.jobs_pointer = Pointer(self)
@@ -145,7 +144,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.proceed_question = ProceedQuestion(self)
         self.job_error_dialog = JobError(self)
         self.keyboard = Manager(self)
-        _gui = self
+        get_gui.ans = self
         self.opts = opts
         self.device_connected = None
         self.gui_debug = gui_debug
@@ -220,7 +219,11 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         opts = self.opts
         self.preferences_action, self.quit_action = actions
         self.library_path = library_path
+        self.library_broker = GuiLibraryBroker(db)
         self.content_server = None
+        self.server_change_notification_timer = t = QTimer(self)
+        self.server_changes = Queue()
+        t.setInterval(1000), t.timeout.connect(self.handle_changes_from_server_debounced), t.setSingleShot(True)
         self._spare_pool = None
         self.must_restart_before_config = False
         self.listener = Listener(listener)
@@ -246,7 +249,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         # Jobs Button {{{
         self.job_manager = JobManager()
         self.jobs_dialog = JobsDialog(self, self.job_manager)
-        self.jobs_button = JobsButton(horizontal=True, parent=self)
+        self.jobs_button = JobsButton(parent=self)
         self.jobs_button.initialize(self.jobs_dialog, self.job_manager)
         # }}}
 
@@ -263,25 +266,23 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.tb_wrapper = textwrap.TextWrapper(width=40)
         self.viewers = collections.deque()
         self.system_tray_icon = None
-        if config['systray_icon']:
+        do_systray = config['systray_icon'] or opts.start_in_tray
+        if do_systray:
             self.system_tray_icon = factory(app_id='com.calibre-ebook.gui').create_system_tray_icon(parent=self, title='calibre')
         if self.system_tray_icon is not None:
             self.system_tray_icon.setIcon(QIcon(I('lt.png', allow_user_override=False)))
             if not (iswindows or isosx):
-                self.system_tray_icon.setIcon(QIcon.fromTheme('calibre-gui', self.system_tray_icon.icon()))
+                self.system_tray_icon.setIcon(QIcon.fromTheme('calibre-tray', self.system_tray_icon.icon()))
             self.system_tray_icon.setToolTip(self.jobs_button.tray_tooltip())
             self.system_tray_icon.setVisible(True)
             self.jobs_button.tray_tooltip_updated.connect(self.system_tray_icon.setToolTip)
-        elif config['systray_icon']:
-            prints('Failed to create system tray icon, your desktop environment probably does not support the StatusNotifier spec')
+        elif do_systray:
+            prints('Failed to create system tray icon, your desktop environment probably'
+                   ' does not support the StatusNotifier spec https://www.freedesktop.org/wiki/Specifications/StatusNotifierItem/')
         self.system_tray_menu = QMenu(self)
         self.toggle_to_tray_action = self.system_tray_menu.addAction(QIcon(I('page.png')), '')
         self.toggle_to_tray_action.triggered.connect(self.system_tray_icon_activated)
         self.system_tray_menu.addAction(self.donate_action)
-        self.donate_button.clicked.connect(self.donate_action.trigger)
-        self.donate_button.setToolTip(self.donate_action.text().replace('&', ''))
-        self.donate_button.setIcon(self.donate_action.icon())
-        self.donate_button.setStatusTip(self.donate_button.toolTip())
         self.eject_action = self.system_tray_menu.addAction(
                 QIcon(I('eject.png')), _('&Eject connected device'))
         self.eject_action.setEnabled(False)
@@ -398,11 +399,12 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         if config['autolaunch_server']:
             self.start_content_server()
 
+        if show_gui:
+            self.show()
         self.read_settings()
 
         self.finalize_layout()
-        if self.bars_manager.showing_donate:
-            self.donate_button.start_animation()
+        self.bars_manager.start_animation()
         self.set_window_title()
 
         for ac in self.iactions.values():
@@ -418,13 +420,17 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
         register_keyboard_shortcuts()
         self.keyboard.finalize()
-        if show_gui:
-            # Note this has to come after restoreGeometry() because of
-            # https://bugreports.qt.io/browse/QTBUG-56831
-            self.show()
         if self.system_tray_icon is not None and self.system_tray_icon.isVisible() and opts.start_in_tray:
             self.hide_windows()
         self.auto_adder = AutoAdder(gprefs['auto_add_path'], self)
+
+        # Now that the gui is initialized we can restore the quickview state
+        # The same thing will be true for any action-based operation with a
+        # layout button
+        from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
+        qv = get_quickview_action_plugin()
+        if qv:
+            qv.qv_button.restore_state()
         self.save_layout_state()
 
         # Collect cycles now
@@ -446,7 +452,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                     'log will be displayed automatically.')%self.gui_debug, show=True)
 
     def esc(self, *args):
-        self.clear_button.click()
+        self.search.clear()
 
     def shift_esc(self):
         self.current_view().setFocus(Qt.OtherFocusReason)
@@ -473,17 +479,50 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.iactions['Connect Share'].set_smartdevice_action_state()
 
     def start_content_server(self, check_started=True):
-        from calibre.library.server.main import start_threaded_server
-        from calibre.library.server import server_config
-        self.content_server = start_threaded_server(
-                self.library_view.model().db, server_config().parse())
+        from calibre.srv.embedded import Server
+        if not gprefs.get('server3_warning_done', False):
+            gprefs.set('server3_warning_done', True)
+            if os.path.exists(os.path.join(config_dir, 'server.py')):
+                try:
+                    os.remove(os.path.join(config_dir, 'server.py'))
+                except EnvironmentError:
+                    pass
+                warning_dialog(self, _('Content server changed!'), _(
+                    'calibre 3 comes with a completely re-written content server.'
+                    ' As such any custom configuration you have for the content'
+                    ' server no longer applies. You should check and refresh your'
+                    ' settings in Preferences->Sharing->Sharing over the net'), show=True)
+        self.content_server = Server(self.library_broker, Dispatcher(self.handle_changes_from_server))
         self.content_server.state_callback = Dispatcher(
                 self.iactions['Connect Share'].content_server_state_changed)
         if check_started:
             self.content_server.start_failure_callback = \
                 Dispatcher(self.content_server_start_failed)
+        self.content_server.start()
+
+    def handle_changes_from_server(self, library_path, change_event):
+        if DEBUG:
+            prints('Received server change event: {} for {}'.format(change_event, library_path))
+        if self.library_broker.is_gui_library(library_path):
+            self.server_changes.put((library_path, change_event))
+            self.server_change_notification_timer.start()
+
+    def handle_changes_from_server_debounced(self):
+        if self.shutting_down:
+            return
+        changes = []
+        while True:
+            try:
+                library_path, change_event = self.server_changes.get_nowait()
+            except Empty:
+                break
+            if self.library_broker.is_gui_library(library_path):
+                changes.append(change_event)
+        if changes:
+            handle_changes(changes, self)
 
     def content_server_start_failed(self, msg):
+        self.content_server = None
         error_dialog(self, _('Failed to start Content server'),
                 _('Could not start the Content server. Error:\n\n%s')%msg,
                 show=True)
@@ -494,7 +533,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
     def create_spare_pool(self, *args):
         if self._spare_pool is None:
-            num = min(detect_ncpus(), int(config['worker_limit']/2.0))
+            num = min(detect_ncpus(), config['worker_limit']//2)
             self._spare_pool = Pool(max_workers=num, name='GUIPool')
 
     def spare_pool(self):
@@ -559,17 +598,34 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         if self.content_server is not None and \
                 self.content_server.exception is not None:
             error_dialog(self, _('Failed to start Content server'),
-                         unicode(self.content_server.exception)).exec_()
+                         unicode_type(self.content_server.exception)).exec_()
 
     @property
     def current_db(self):
         return self.library_view.model().db
 
+    def refresh_all(self):
+        m = self.library_view.model()
+        m.db.data.refresh(clear_caches=False, do_search=False)
+        self.saved_searches_changed(recount=False)
+        m.resort()
+        m.research()
+        self.tags_view.recount()
+
+    def handle_cli_args(self, args):
+        if isinstance(args, string_or_bytes):
+            args = [args]
+        files = [os.path.abspath(p) for p in args if not os.path.isdir(p) and os.access(p, os.R_OK)]
+        if files:
+            self.iactions['Add Books'].add_filesystem_book(files)
+
     def another_instance_wants_to_talk(self):
         try:
             msg = self.listener.queue.get_nowait()
         except Empty:
             return
+        if isinstance(msg, bytes):
+            msg = msg.decode('utf-8', 'replace')
         if msg.startswith('launched:'):
             import json
             try:
@@ -583,9 +639,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                                  det_msg='Invalid msg: %r' % msg, show=True)
                 argv = ()
             if isinstance(argv, (list, tuple)) and len(argv) > 1:
-                files = [os.path.abspath(p) for p in argv[1:] if not os.path.isdir(p) and os.access(p, os.R_OK)]
-                if files:
-                    self.iactions['Add Books'].add_filesystem_book(files)
+                self.handle_cli_args(argv[1:])
             self.setWindowState(self.windowState() & ~Qt.WindowMinimized|Qt.WindowActive)
             self.show_windows()
             self.raise_()
@@ -593,10 +647,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         elif msg.startswith('refreshdb:'):
             m = self.library_view.model()
             m.db.new_api.reload_from_db()
-            m.db.data.refresh(clear_caches=False, do_search=False)
-            m.resort()
-            m.research()
-            self.tags_view.recount()
+            self.refresh_all()
         elif msg.startswith('shutdown:'):
             self.quit(confirm_quit=False)
         elif msg.startswith('bookedited:'):
@@ -613,8 +664,25 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             except Exception:
                 import traceback
                 traceback.print_exc()
+        elif msg.startswith('web-store:'):
+            import json
+            try:
+                data = json.loads(msg[len('web-store:'):])
+            except ValueError:
+                prints('Failed to decode message from other instance: %r' % msg)
+            path = data['path']
+            if data['tags']:
+                before = self.current_db.new_api.all_book_ids()
+            self.iactions['Add Books'].add_filesystem_book([path], allow_device=False)
+            if data['tags']:
+                db = self.current_db.new_api
+                after = self.current_db.new_api.all_book_ids()
+                for book_id in after - before:
+                    tags = list(db.field_for('tags', book_id))
+                    tags += list(data['tags'])
+                    self.current_db.new_api.set_field('tags', {book_id: tags})
         else:
-            print msg
+            prints('Ignoring unknown message from other instance: %r' % msg[:20])
 
     def current_view(self):
         '''Convenience method that returns the currently visible view '''
@@ -631,81 +699,74 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     def booklists(self):
         return self.memory_view.model().db, self.card_a_view.model().db, self.card_b_view.model().db
 
-    def library_moved(self, newloc, copy_structure=False, call_close=True,
-            allow_rebuild=False):
+    def library_moved(self, newloc, copy_structure=False, allow_rebuild=False):
         if newloc is None:
             return
-        default_prefs = None
-        try:
-            olddb = self.library_view.model().db
-            if copy_structure:
-                default_prefs = olddb.prefs
-        except:
-            olddb = None
-        if copy_structure and olddb is not None and default_prefs is not None:
-            default_prefs['field_metadata'] = olddb.new_api.field_metadata.all_metadata()
-        try:
-            db = LibraryDatabase(newloc, default_prefs=default_prefs)
-        except apsw.Error:
-            if not allow_rebuild:
-                raise
-            import traceback
-            repair = question_dialog(self, _('Corrupted database'),
-                    _('The library database at %s appears to be corrupted. Do '
-                    'you want calibre to try and rebuild it automatically? '
-                    'The rebuild may not be completely successful.')
-                    % force_unicode(newloc, filesystem_encoding),
-                    det_msg=traceback.format_exc()
-                    )
-            if repair:
-                from calibre.gui2.dialogs.restore_library import repair_library_at
-                if repair_library_at(newloc, parent=self):
-                    db = LibraryDatabase(newloc, default_prefs=default_prefs)
-                else:
-                    return
-            else:
-                return
-        if self.content_server is not None:
-            self.content_server.set_database(db)
-        self.library_path = newloc
-        prefs['library_path'] = self.library_path
-        self.book_on_device(None, reset=True)
-        db.set_book_on_device_func(self.book_on_device)
-        self.library_view.set_database(db)
-        self.tags_view.set_database(db, self.alter_tb)
-        self.library_view.model().set_book_on_device_func(self.book_on_device)
-        self.status_bar.clear_message()
-        self.search.clear()
-        self.saved_search.clear()
-        self.book_details.reset_info()
-        # self.library_view.model().count_changed()
-        db = self.library_view.model().db
-        self.iactions['Choose Library'].count_changed(db.count())
-        self.set_window_title()
-        self.apply_named_search_restriction('')  # reset restriction to null
-        self.saved_searches_changed(recount=False)  # reload the search restrictions combo box
-        if db.prefs['virtual_lib_on_startup']:
-            self.apply_virtual_library(db.prefs['virtual_lib_on_startup'])
-        self.rebuild_vl_tabs()
-        for action in self.iactions.values():
-            action.library_changed(db)
-        if olddb is not None:
+        with self.library_broker:
+            default_prefs = None
             try:
-                if call_close:
-                    olddb.close()
+                olddb = self.library_view.model().db
+                if copy_structure:
+                    default_prefs = olddb.prefs
             except:
-                import traceback
-                traceback.print_exc()
-            olddb.break_cycles()
-        if self.device_connected:
-            self.set_books_in_library(self.booklists(), reset=True)
-            self.refresh_ondevice()
-            self.memory_view.reset()
-            self.card_a_view.reset()
-            self.card_b_view.reset()
-        self.set_current_library_information(current_library_name(), db.library_id,
-                                             db.field_metadata)
-        self.library_view.set_current_row(0)
+                olddb = None
+            if copy_structure and olddb is not None and default_prefs is not None:
+                default_prefs['field_metadata'] = olddb.new_api.field_metadata.all_metadata()
+            db = self.library_broker.prepare_for_gui_library_change(newloc)
+            if db is None:
+                try:
+                    db = LibraryDatabase(newloc, default_prefs=default_prefs)
+                except apsw.Error:
+                    if not allow_rebuild:
+                        raise
+                    import traceback
+                    repair = question_dialog(self, _('Corrupted database'),
+                            _('The library database at %s appears to be corrupted. Do '
+                            'you want calibre to try and rebuild it automatically? '
+                            'The rebuild may not be completely successful.')
+                            % force_unicode(newloc, filesystem_encoding),
+                            det_msg=traceback.format_exc()
+                            )
+                    if repair:
+                        from calibre.gui2.dialogs.restore_library import repair_library_at
+                        if repair_library_at(newloc, parent=self):
+                            db = LibraryDatabase(newloc, default_prefs=default_prefs)
+                        else:
+                            return
+                    else:
+                        return
+            self.library_path = newloc
+            prefs['library_path'] = self.library_path
+            self.book_on_device(None, reset=True)
+            db.set_book_on_device_func(self.book_on_device)
+            self.library_view.set_database(db)
+            self.tags_view.set_database(db, self.alter_tb)
+            self.library_view.model().set_book_on_device_func(self.book_on_device)
+            self.status_bar.clear_message()
+            self.search.clear()
+            self.saved_search.clear()
+            self.book_details.reset_info()
+            # self.library_view.model().count_changed()
+            db = self.library_view.model().db
+            self.iactions['Choose Library'].count_changed(db.count())
+            self.set_window_title()
+            self.apply_named_search_restriction('')  # reset restriction to null
+            self.saved_searches_changed(recount=False)  # reload the search restrictions combo box
+            if db.prefs['virtual_lib_on_startup']:
+                self.apply_virtual_library(db.prefs['virtual_lib_on_startup'])
+            self.rebuild_vl_tabs()
+            for action in self.iactions.values():
+                action.library_changed(db)
+            self.library_broker.gui_library_changed(db, olddb)
+            if self.device_connected:
+                self.set_books_in_library(self.booklists(), reset=True)
+                self.refresh_ondevice()
+                self.memory_view.reset()
+                self.card_a_view.reset()
+                self.card_b_view.reset()
+            self.set_current_library_information(current_library_name(), db.library_id,
+                                                db.field_metadata)
+            self.library_view.set_current_row(0)
         # Run a garbage collection now so that it does not freeze the
         # interface later
         gc.collect()
@@ -721,7 +782,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             font.setBold(True)
             font.setItalic(True)
         self.virtual_library.setFont(font)
-        title = u'{0} - || {1}{2} ||'.format(
+        title = '{0} - || {1}{2} ||'.format(
                 __appname__, self.iactions['Choose Library'].library_name(), restrictions)
         self.setWindowTitle(title)
 
@@ -780,7 +841,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                 files that are large undifferentiated blocks of text, this
                 splitting fails.
                 <p>You can <b>work around the problem</b> by either increasing the
-                maximum split size under EPUB Output in the conversion dialog,
+                maximum split size under <i>EPUB output</i> in the conversion dialog,
                 or by turning on Heuristic Processing, also in the conversion
                 dialog. Note that if you make the maximum split size too large,
                 your e-book reader may have trouble with the EPUB.
@@ -844,7 +905,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             pass
         if not minz:
             self.job_error_dialog.show_error(dialog_title,
-                    _('<b>Failed</b>')+': '+unicode(job.description),
+                    _('<b>Failed</b>')+': '+unicode_type(job.description),
                     det_msg=job.details, retry_func=retry_func)
 
     def read_settings(self):
@@ -858,6 +919,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             config.set('main_window_geometry', self.saveGeometry())
             dynamic.set('sort_history', self.library_view.model().sort_history)
             self.save_layout_state()
+            self.stack.tb_widget.save_state()
 
     def quit(self, checked=True, restart=False, debug_on_restart=False,
             confirm_quit=True):
@@ -871,10 +933,14 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             pass
         self.restart_after_quit = restart
         self.debug_on_restart = debug_on_restart
+        if self.system_tray_icon is not None and self.restart_after_quit:
+            # Needed on windows to prevent multiple systray icons
+            self.system_tray_icon.setVisible(False)
         QApplication.instance().quit()
 
     def donate(self, *args):
-        open_url(QUrl('https://calibre-ebook.com/donate'))
+        from calibre.utils.localization import localize_website_link
+        open_url(QUrl(localize_website_link('https://calibre-ebook.com/donate')))
 
     def confirm_quit(self):
         if self.job_manager.has_jobs():
@@ -906,6 +972,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     def shutdown(self, write_settings=True):
         self.shutting_down = True
         self.show_shutdown_message()
+        self.server_change_notification_timer.stop()
 
         from calibre.customize.ui import has_library_closed_plugins
         if has_library_closed_plugins():
@@ -932,7 +999,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         if write_settings:
             self.write_settings()
         self.check_messages_timer.stop()
-        if hasattr(self, 'update_checker'):
+        if getattr(self, 'update_checker', None):
             self.update_checker.shutdown()
         self.listener.close()
         self.job_manager.server.close()
